#include <stdio.h>
#include <dali_w32.h>

//These are default values

//DALI_IP is the IP of the remote DALI server.  
#define DALI_IP   "127.0.0.1"

//DALI_PORT is the port of the remote DALI server.
#define DALI_PORT 8444

//DALI_DEMO is the name of the remote board to access.
#define DALI_DEMO "DEMO"


#define DIO_PORT 0
#define DIO_BIT  0

//MY_PORT is the local port that the DALI server will talk back to for alarm notification.
//If the DALI server cannot communicate back to your machine at this port (due to firewall, etc)
//then alarms will not work.
#define MY_PORT 17391


char* dali_ip;
int   dali_port;
char* dali_board;

CDali dali;
UINT demo;
UINT alarm_handle;
int val;

void HandleError(int rc, char* str)
{
	if (rc)
    {
        printf("%s Error [%d]: %s\n", str, rc, dali.GetLastErrorString());
		getchar();
        exit(rc);
    }
}

UINT alarm_callback(int code, DALI_DATA* data)
{
	switch (code)
	{
	case DALI_CMD_REGISTERED:
		printf("Successfully registered with server\n");
		break;

	case DALI_CMD_DIO_IN_BIT:
		printf("DIO-In alarm!  Port: %d  Bit: %d  Value: %d\n", data->low_channel_port, data->high_channel_bit, data->values[0]);
		break;

	default:
		printf("Received unexpected code: %X; val: %d\n", code, data->values[0]);
	}

	return 0;
}


int main(int argc, char ** argv)
{
	//Get command line args.
	//Usage: ConsoleServerInfo <remote ip> <remote port> <board name>

	dali_ip =    argc > 1 ? argv[1] : DALI_IP;
	dali_port =  argc > 2 ? atoi(argv[2]) : DALI_PORT;
	dali_board = argc > 3 ? argv[3] : DALI_DEMO;


	//First you must initialize the CDali object with a server IP, port and secure flag.
	//Setting the secure argument to '1' will make the CDali object use SSL (https://)
    
	printf("Server: %s\n", dali_ip);
	printf("Port:   %d\n\n", dali_port);
	
    HandleError(dali.SetServer(dali_ip, dali_port, 0), "dali.SetServer()");

	
	//Now we must get the handle for the "DEMO" board.  The name of this board can be
	//modified in the defines above - it *must* match the name of the board on the server
	//you wish to communicate with.

	//Please remember that GetBoardHandle does more than just return a handle.  It allocates
	//and initializes data in the CDali object related to the board, and must be called at
	//least once before any operations that require the returned board handle.
	
	HandleError(dali.GetBoardHandle(dali_board, &demo), "dali.GetBoardHandle()");



	//Now to fill out the DALI_INTERFACE_SETTINGS struct for the alarm handler.

	DALI_INTERFACE_SETTINGS dis;

	dis.dali_board = demo;                      //use the 'demo' board handle
	dis.operation_type = DALI_CMD_DIO_RB_ALARM; //we're going to alarm on DIO changes
	dis.low_channel_port = DIO_PORT;            //specify the DIO port
	dis.high_channel_bit = DIO_BIT;             //and the DIO bit
	dis.port = MY_PORT;                         //which port to bind to locally.
	dis.alarm.period = 1;                       //scan for changes every 'period' seconds
	dis.alarm.duration = 60;                    //for 'duration' seconds
	dis.alarm.postalarmpause = 1;               //how many seconds after an alarm to start checking for another

	//If we were doing an A/D alarm (DALI_CMD_AD_ALARM) we would also have to fill out
	//dis.low_value and dis.high_value for the A/D range to alarm on.

	//Now we will register the alarm interface and print out the handle to the alarm

	HandleError(dali.RegisterInterface(&dis, alarm_callback, &alarm_handle), "dali.RegisterInterface()");

	printf("Alarm interface is regsitered with handle: %d\n", alarm_handle);




	//Now we will toggle the DIO pin that the alarm is monitoring.  If the DIO pin started
	//high, there will be no alarm after the first operation.

	getchar();	

	HandleError(dali.PerformDIOOutputBit(demo, DIO_PORT, DIO_BIT, 1), "dali.PerformDIOOutputBit()");
	HandleError(dali.PerformDIOInputBit(demo, DIO_PORT, DIO_BIT, &val), "dali.PerformDIOInputBit()");
	printf("val: %d\n", val);

	getchar();

	HandleError(dali.PerformDIOOutputBit(demo, DIO_PORT, DIO_BIT, 0), "dali.PerformDIOOutputBit()");
	HandleError(dali.PerformDIOInputBit(demo, DIO_PORT, DIO_BIT, &val), "dali.PerformDIOInputBit()");
	printf("val: %d\n", val);

	getchar();
	
	HandleError(dali.PerformDIOOutputBit(demo, DIO_PORT, DIO_BIT, 1), "dali.PerformDIOOutputBit()");
	HandleError(dali.PerformDIOInputBit(demo, DIO_PORT, DIO_BIT, &val), "dali.PerformDIOInputBit()");
	printf("val: %d\n", val);

	getchar();


	//Now we will unregister the alarm and toggle the DIO again.  There should be no callback.
	
	HandleError(dali.UnRegisterInterface(alarm_handle), "dali.UnRegisterInterface()");
	printf("The alarm has been unregistered\n\n");

	HandleError(dali.PerformDIOOutputBit(demo, DIO_PORT, DIO_BIT, 0), "dali.PerformDIOOutputBit()");
	HandleError(dali.PerformDIOInputBit(demo, DIO_PORT, DIO_BIT, &val), "dali.PerformDIOInputBit()");
	printf("val: %d\n", val);

	getchar();
	
	HandleError(dali.PerformDIOOutputBit(demo, DIO_PORT, DIO_BIT, 1), "dali.PerformDIOOutputBit()");
	HandleError(dali.PerformDIOInputBit(demo, DIO_PORT, DIO_BIT, &val), "dali.PerformDIOInputBit()");
	printf("val: %d\n", val);

	getchar();



	return 0;
}