/*
 * DALI C/C++ Client
 * Copyright (c) Diamond Systems Corporation 2002
 * All Rights Reserved
 *
 * http://www.diamondsystems.com
 *
 */

/*
 * This program implements a simple "DSCUD-like" layer on top of gSOAP to
 * allow C/C++ users to have a simple way of accessing DALI from any 
 * platform supported by gSOAP.  
 */ 


#ifndef __DALIC_H_
#define __DALIC_H_

#ifdef __cplusplus
extern "C" {
#endif

/* DALI Errors */
#define DALI_OK 0 					/* No error */
#define DALI_ERROR 1  				/* General error */
#define DALI_SERVER_ERROR 2 		/* Error in DALI server */
#define DALI_BAD_REQUEST 3  		/* Error in DALI client */
#define DALI_BAD_RESPONSE 4  		/* Corrupt SOAP response from server */
#define DALI_CONNECTION_FAILED 5  	/* Unable to connect to server */
#define DALI_SSL_ERROR 6  			/* General SSL error */
#define DALI_HTTP_ERROR 7 			/* General HTTP error */
#define DALI_TCP_BIND_ERROR 8  		/* Unable to bind to port for alarm */
#define DALI_TCP_ACCEPT_ERROR 9  	/* Unable to accept alarm request */

#define DBUFSIZE 128

/* The "handle" to the DALI client */
typedef struct {
	char *service; 			/* URL of the DALI server.  See the DALI docs for
							 * more information */
	char *userid; 			/* Username for authenticated requests */
	char *password; 		/* Password for authenticated requests */
	char last_error[DBUFSIZE];  	/* Holds last error string */
	int   thread_safe;  	/* Set to 1 for mutex locking */

	/* For internal use only.  */
	void* mutex;
	void *soap;
	void *alarm_soap;
} DALIENV;

/* Returned in BoardInfo() to indicate which board is supported */
enum daliboardtype { 
	DALI_BOARD_UNKNOWN, /* DALI server returned an unknown board */
	DALI_DMM, 
	DALI_DMM32, 
	DALI_DMMAT, 
	DALI_DMM16AT, 
	DALI_PROM, 
	DALI_ZMM, 
	DALI_TEST /* Special "virtual" board for testing */
};

/* Returned in ChannelInfo() to indicate which channel type is used */
enum dalichtype { 
	DALI_CH_UNKNOWN, /* DALI server returned an unkown channel type */
	DALI_CH_AD, 
	DALI_CH_DA, 
	DALI_CH_DIOIN, 
	DALI_CH_DIOOUT 
};

/* Returned in ChannelInfo() to indicate the channel security level */
enum dalisecurity { 
	DALI_CH_DISABLED,  /* The channel is temporarily disabled */
	DALI_PUBLIC, 
	DALI_MUST_LOGIN  /* Requests for this channel must be authenticated
					  * See DALIENV.user and DALIENV.password */
};

/* Returned by BoardInfo() one for each board operated by DALI */
typedef struct {
	char board[DBUFSIZE]; 				/* The board alias */
	enum daliboardtype boardtype;	/* See above for enum definition */
	int baseaddress; 				/* Base I/O address of board */
} DALIBOARD;

/* Returned by ChannelInfo() one for each channel supported by a board
 * operated by DALI */
typedef struct {
	int channel; 				/* Channel number */
	enum dalichtype type; 		/* See above for enum definition */
	enum dalisecurity security;	/* See above for enum definition */
} DALICHANNEL;

/* Use to set DALIADSETTINGS.polarity */
#define DALI_BIPOLAR  0
#define DALI_UNIPOLAR 1

/* Passed to all AD calls to specify AD board settings for the samples
 * taken in response to the call. */
typedef struct {
	int range; /* Valid values: 5 or 10 */
	int gain; /* Valid values: 1 2 4 or 8 */
	int polarity; /* See DALI_BIPOLAR and DALI_UNIPOLAR */
} DALIADSETTINGS;

/* Use to set the DALIALARM.actiontype */
#define DALI_ACTION_SOAP "SOAP"
#define DALI_ACTION_EMAIL "EMAIL"
#define DALI_ACTION_LOG "LOG"

/* Passed to all DALI Alarm calls to configure alarm parameters */
typedef struct {
	char actiontype[DBUFSIZE]; 		/* What the server should do when an alarm occurs */
	char actiontarget[DBUFSIZE];  	/* Parameter for action */
	char message[DBUFSIZE];  		/* Explanatory text for alarm action messages */
	int period; 			/* Hz the server should check the condition */
	int duration; 			/* Lifetime in seconds of the alarm before it is 
							 * deleted */
	int postalarmpause;	    /* Pause in seconds after the server issues an 
							 * alarm before it will issue another */
} DALIALARMSETTINGS;

/* Returned when an alarm notice is received */
typedef struct {
	char id[DBUFSIZE];      /* The alarm ID */
	char service[DBUFSIZE]; /* The service of the DALI server that sent the alarm */
	char message[DBUFSIZE]; /* Text message describing the alarm condition */
	int value;              /* The value that triggered the alarm */
	int status;             /* Status code, non zero on error */
	long timestamp;         /* Timestamp when the alarm was generated */
} DALIALARMNOTICE;

typedef struct {
	char id[DBUFSIZE];
	char board[DBUFSIZE];
	char actiontype[DBUFSIZE];
	char actiontarget[DBUFSIZE];
	char message[DBUFSIZE];
	int period;
	int duration;
	long expires;
	int postalarmpause;
	int channel;
	int bit;
	char channeltype[DBUFSIZE];
	char alarmtype[DBUFSIZE];
	int lowvalue;
	int highvalue;
} DALIALARM;

/* Initialization and deinitialization functions */
int DALIInit(DALIENV *env);
void DALIFree(DALIENV *env);

/* Basic AD I/O */
int DALIADSample(DALIENV *env, int channel, DALIADSETTINGS *settings,
				int *value);
int DALIADScan(DALIENV *env, int low_channel, int high_channel, 
				DALIADSETTINGS *settings, int *values);

int DALIADSampleMulti(DALIENV *env, int channel, int count, float hz, 
				DALIADSETTINGS *settings, int *values);
int DALIADScanMulti(DALIENV *env, int low_channel, int high_channel, 
				int count, float hz, DALIADSETTINGS *settings, int *values);

/* Basic DA I/O */
int DALIDAConvert(DALIENV *env, int channel, int value);
int DALIDAConvertScan(DALIENV *env, int low_channel, int high_channel, 
				int *values);

/* Read back the last value set on DA channels by DALI */
int DALIDAReadback(DALIENV *env, int channel, int *value);
int DALIDAReadbackScan(DALIENV *env, int low_channel, int high_channel,
				int *values);

/* Basic Digital I/O */
int DALIDIOOutputByte(DALIENV *env, int channel, int value);
int DALIDIOOutputBit(DALIENV *env, int channel, int bit, int value);
int DALIDIOInputByte(DALIENV *env, int channel, int *value);
int DALIDIOInputBit(DALIENV *env, int channel, int bit, int *value);

/* Read back the last value set on DIO channels by DALI */
int DALIDIOReadbackByte(DALIENV *env, int channel, int *value);
int DALIDIOReadbackBit(DALIENV *env, int channel, int bit, int *value);

/* Get information about boards and channels operated by DALI */
int DALIGetBoardInfo(DALIENV *env, DALIBOARD **boards, int *board_count);
int DALIGetChannelInfo(DALIENV *env, DALICHANNEL **channels, int *channel_count);
int DALIGetAlarmInfo(DALIENV *env, DALIALARM **alarms, int *alarm_count);

/* "ping" the server to make sure it is alive and working */
int DALIPing(DALIENV *env, int *result);

/* Alarm functions */
int DALIAlarmADOutOfRange(DALIENV *env, int channel, int low_value, 
				int high_value, DALIADSETTINGS *settings, 
				DALIALARMSETTINGS *alarm, char *alarmid, int alarmid_bufsize);
int DALIAlarmDAOnChange(DALIENV *env, int channel, DALIALARMSETTINGS *alarm, 
				char *alarmid, int alarmid_bufsize);
int DALIAlarmDIOInOnChange(DALIENV *env, int channel, int bit, 
				DALIALARMSETTINGS *alarm, char *alarmid, int alarmid_bufsize);
int DALIAlarmDIOOutOnChange(DALIENV *env, int channel, int bit, 
				DALIALARMSETTINGS *alarm, char *alarmid, int alarmid_bufsize);
int DALIInitAlarmListener(DALIENV *env, int port);
int DALIWaitForAlarm(DALIENV *env, DALIALARMNOTICE *alarm_notice);
int DALIAlarmIsAlive(DALIENV *env, char *alarmid);
int DALIFreeAlarmListener(DALIENV *env);
int DALIUnregisterAlarm(DALIENV *env, char *alarmid);

#ifdef __cplusplus
}
#endif

#endif
