/*

	dali_w32.cpp
	
	This is the main CDali class implementation file.

	(c) Diamond Systems Corporation 2003

	http://www.diamondsystems.com

	(c) Copyright 2003 Diamond Systems Corporation. Use of this source code
	is subject to the terms of Diamond Systems' Software License Agreement.
	Diamond Systems provides no warranty of proper performance if this
	source code is modified.

	
*/

#include "dali_w32.h"
#include "dalic.h"
#include <winsock.h>

char* DaliError(int rc)
{
	switch (rc)
	{
	case DALI_OK: return "No error";
	case DALI_ERROR: return "General error";
	case DALI_SERVER_ERROR: return "Error in DALI server";
	case DALI_BAD_REQUEST: return "Bad SOAP request";
	case DALI_BAD_RESPONSE: return "Bad SOAP response";
	case DALI_CONNECTION_FAILED: return "Unable to connect to server";
	case DALI_SSL_ERROR: return "General SSL error";
	case DALI_HTTP_ERROR: return "General HTTP error";
	case DALI_TCP_BIND_ERROR: return "Unable to bind to port for alarm";
	case DALI_TCP_ACCEPT_ERROR: return "Unable to accept alarm request";
	default: return "Unknown Error";
	}
}

//PQThread is the wrapper to call the CDali ProcessPQ member function as a seperate thread.
//This is necessary because member functions should not be used as thread functions.
ULONG __stdcall PQThread(void* cdali)
{
	((CDali*)cdali)->ProcessPQ();
	return 0;
}

//ATThread is the wrapper to call the CDali AlertThread member function as a seperate thread.
ULONG __stdcall ATThread(void* cdali)
{
	((CDali*)cdali)->AlertThread();
	return 0;
}

//The CDali constructor.  Used to initialize variables and retrieve network information.
CDali::CDali()
{
	server[0] = 0;
	port = 0;
	alive = 1;

	userid[0] = 0;
	passwd[0] = 0;

	numboards = 0;
	numifaces = 0;

	run_at = 0;
	run_pq = 1;
	
	my_port = -1;

	at_handle = 0;
	pq_handle = 0;

	for (int x = 0; x < DALI_MAX_BOARDS; x++)
	{
		chans[x] = NULL;	
		boards[x].board[0] = 0;
		boards[x].baseaddress = -1;
	}

	DeleteChannelInfo();
	
	for (int i = 0; i < DALI_MAX_INTERFACES; i++)
	{
		ifaces[i].registered = FALSE;
		ifaces[i].active = FALSE;
	}

	//This section here gets the IP address of the host.
	WSADATA wsaData;
	WSAStartup(MAKEWORD(1, 1), &wsaData);
	
	char ip[256];
	unsigned long sz = 256;	
	GetComputerName(ip, &sz);  //Get computer name
	struct hostent *h;
	h=gethostbyname(ip);  //Get host by name
	
	int ip_pos = 0;	
	while (h->h_addr_list[ip_pos] != 0)
	{
		//store next ip in list to my_ip
		strcpy(my_ip, inet_ntoa(*((struct in_addr *)h->h_addr_list[ip_pos])));
		//if IP = loopback, go to the next one.
		if (strcmp(my_ip, "127.0.0.1"))
			break;
		ip_pos++;
	}

	WSACleanup();

	//initialize ENV (dalic.lib) - thread safe
	DALIInit(&env);
	env.thread_safe = 1;
}

//Same constructor, but sets the username, password and secure http settings
CDali::CDali(LPCTSTR name, UINT port, LPCTSTR userid, LPCTSTR passwd, BOOL secure_http)
{	
	CDali();

	strncpy(this->userid, userid, DALI_MAX_USERID_LEN);
	strncpy(this->passwd, passwd, DALI_MAX_PASSWD_LEN);

	SetServer(name, port, secure_http);
}

//Deconstructor
CDali::~CDali()
{
	alive = 0;  //Set the alive flag to false.  This will alert other threads to shut down.

	//unregister/unactivate all interfaces
	for (int i = 0; i < DALI_MAX_INTERFACES; i++)
	{
		if (ifaces[i].registered)
		{
			if (UnRegisterInterface(i)) //This signals interface that it is unregistered
				break;
		}

		ifaces[i].registered = 0;
		ifaces[i].active = 0;
	}

	//Kill the alarm listener (dalic.lib)
	//This will close the socket and trigger an error.  The error will be detected by the AT thread
	//and it will stop.
	if (run_at)
		DALIFreeAlarmListener(&env);
	
	run_at = 0;
	run_pq = 0;

	//Once run_pq is set to 0, the PQ thread should perform only one more sample, at which time it will
	//see that run_pq == 0 and shut down
	
	if (at_handle) //If the alarm thread is running, we'll wait for it to stop
	{
		DWORD retval = 259;
		while (retval != 0)
		{
			GetExitCodeThread(at_handle, &retval);			
			Sleep(10);
		}
	}

	if (pq_handle) //If the priority queue thread is running, we'll wait for it to stop.
	{
		DWORD retval = 259;
		while (retval != 0)
		{
			GetExitCodeThread(pq_handle, &retval);
			Sleep(10);
		}
	}

	//Free up memory
	DeleteChannelInfo();	
	DALIFree(&env);
	
}


UINT CDali::SetServer(LPCTSTR name, UINT port, BOOL secure_http)
{	
	if (name == NULL)
		return SetLastError(DALI_ERR_INVALID_PARAM, "Server name cannot be NULL");

	if (port > 65535)
		return SetLastError(DALI_ERR_INVALID_PARAM, "Port must be less than 65536");

	//Perform interface/thread shutdown if server is already up.  This is similar to decontructor
	if (server[0] != 0)
	{
		alive = 0;

		for (int i = 0; i < DALI_MAX_INTERFACES; i++)
		{
			if (ifaces[i].registered)
			{
				if (UnRegisterInterface(i))
					break;
			}

			ifaces[i].registered = 0;
			ifaces[i].active = 0;
		}

		if (run_at)
			DALIFreeAlarmListener(&env);
		
		run_at = 0;
		run_pq = 0;
		
		if (at_handle)
		{
			DWORD retval = 259;
			while (retval != 0)
			{
				GetExitCodeThread(at_handle, &retval);
				Sleep(10);
			}
		}

		if (pq_handle)
		{
			DWORD retval = 259;
			while (retval != 0)
			{
				GetExitCodeThread(pq_handle, &retval);
				Sleep(10);
			}
		}

		DeleteChannelInfo();	

		for (int x = 0; x < DALI_MAX_BOARDS; x++)
		{
			chans[x] = NULL;	
			boards[x].board[0] = 0;
			boards[x].baseaddress = -1;
		}

		numifaces = 0;
	}

	//Set server variable
	strncpy(server, name, DALI_MAX_SERVER_SIZE);
	server[DALI_MAX_SERVER_SIZE-1] = 0;

	//Set server_soap address variable (http://... construction)
	_snprintf(server_soap, DALI_MAX_SERVER_SIZE, "http%s://%s:%d/dali/soap", secure_http ? "s" : "", server, port);

	this->port = port;

	env.service = server_soap;
	int res;
	conn_est = DALIPing(&env, &res) == 0 ? 1 : 0;

	return UpdateServerInfo();
}

UINT CDali::GetServer(LPTSTR name,  UINT max_name, UINT* port)
{
	strncpy(name, server, DALI_MAX_SERVER_SIZE);
	*port = this->port;

	return DALI_ERR_NONE;
}

//This function retrieves all server information (boards, channels, etc.)
UINT CDali::UpdateServerInfo()
{		
	env.userid = userid;
	env.password = passwd;
	env.service = server_soap;
	env.thread_safe = 1;

	DALIBOARD* tempboards = NULL;
	int t_numboards;

	//First, get the board info (# boards, name of boards, etc.)
	if (DALIGetBoardInfo(&env, &tempboards, (int*)&t_numboards))
	{		
		return SetLastError(DALI_ERR_CONNECTIONFAIL, "Connection failure while reading board info");
	}
	
	//For each boards, get more detailed information
	for (int b = 0; b < t_numboards; b++)
	{		
		for (UINT c_b = 0; c_b < numboards; c_b++)
		{
			if (!stricmp(boards[c_b].board, tempboards[b].board))
			{
				memcpy(&boards[c_b], &tempboards[b], sizeof(DALIBOARD));
				break;
			}
		}
		
		if (c_b == numboards)
		{
			memcpy(&boards[c_b], &tempboards[b], sizeof(DALIBOARD));
			numboards++;
		}
	}


	if (tempboards) 
		free(tempboards);

	//Get the channel info for each board
	for (UINT i = 0; i < numboards; i++)
	{		
		sprintf(board_soaps[i], "%s/%s", server_soap, boards[i].board);
		env.service = board_soaps[i];

		DALICHANNEL* tempchan = NULL;
		int t_numchans;

		//this is changed to allow possiblity that user registered an unknown board
		if (DALIGetChannelInfo(&env, &tempchan, (int*)&t_numchans))
		{		
			//DeleteChannelInfo();
			//numboards = 0;
			//return SetLastError(DALI_ERR_CONNECTIONFAIL, "Connection failure while reading board info");
		}
		else
		{
			chans[i] = new DALICHANNEL[t_numchans];
			numchans[i] = t_numchans;
			memcpy(chans[i], tempchan, t_numchans * (sizeof DALICHANNEL));
		}
		
		if (tempchan)
			free(tempchan);
	}

	return DALI_ERR_NONE;
}

//Return the integer index of a specified board.  If the board does not exist, it will make
//a placeholder index for the board string.
UINT CDali::GetBoardHandle(LPCTSTR board_name, UINT* board)
{
	//Search for the board string
	for (UINT i = 0; i < numboards; i++)
	{
		if (!strcmp(boards[i].board, board_name))
		{
			*board = i;
			return DALI_ERR_NONE;
		}
	}

	if (numboards == DALI_MAX_BOARDS)
		return SetLastError(DALI_ERR_TOOMANYBOARDS, "Too many registered boards");

	//Now assign the index's board_soaps member to the appropriate url
	*board = numboards;
	strncpy(boards[numboards].board, board_name, DALI_MAX_SERVER_SIZE);
	sprintf(board_soaps[numboards], "%s/%s", server_soap, boards[numboards].board);
	numboards++; //increment the number of boards
	return DALI_ERR_NONE;
}

//Just return the local IP.  This is a utility function
UINT CDali::GetLocalIP(LPTSTR ip)
{
	if (ip == NULL)
		return SetLastError(DALI_ERR_INVALID_PARAM, "Pointer cannot be null");

	strcpy(ip, my_ip);
	return DALI_ERR_NONE;
}

LPCTSTR CDali::GetLastErrorString()
{
	return last_error;
}

UINT CDali::SetUserID(LPCTSTR userid)
{
	strncpy(this->userid, userid, DALI_MAX_USERID_LEN);
	this->userid[DALI_MAX_USERID_LEN-1] = 0;
	
	return DALI_ERR_NONE;
}

UINT CDali::GetUserID(LPTSTR userid, UINT max_len)
{
	strncpy(userid, this->userid, max_len);
	return DALI_ERR_NONE;
}

UINT CDali::SetPassword(LPCTSTR passwd)
{
	strncpy(this->passwd, passwd, DALI_MAX_USERID_LEN);
	this->passwd[DALI_MAX_PASSWD_LEN-1] = 0;

	return DALI_ERR_NONE;
}

UINT CDali::GetPassword(LPTSTR passwd, UINT max_len)
{
	strncpy(passwd, this->passwd, max_len);
	return DALI_ERR_NONE;
}


UINT CDali::GetNumBoards(UINT* numboards)
{
	if (UpdateServerInfo())
		return SetLastError(DALI_ERR_CONNECTIONFAIL, "Could not connect to server");
	
	*numboards = this->numboards;

	return DALI_ERR_NONE;
}

//The next three functions are all the same.  First they check to see if the board information is
//defined (baseaddress != -1).  If not, update the server info.  If the address is still -1, there
//is an error (board does not exist.)  otherwise return the desired information.

UINT CDali::GetBoardInfo(UINT board, DALIBOARD* boardinfo)
{
	if (board < 0 || board >= numboards)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board handle out of range");

	if (boards[board].baseaddress == -1)
	{
		if (UpdateServerInfo())
			return SetLastError(DALI_ERR_CONNECTIONFAIL, "Could not connect to server");
	}

	if (boards[board].baseaddress == -1)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board does not exist on server");

	memcpy(boardinfo, &boards[board], sizeof(DALIBOARD));
	return DALI_ERR_NONE;
}

UINT CDali::GetNumChannels(UINT board, UINT* numchan)
{
	if (board < 0 || board >= numboards)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board handle out of range");

	if (boards[board].baseaddress == -1)
	{
		if (UpdateServerInfo())
			return SetLastError(DALI_ERR_CONNECTIONFAIL, "Could not connect to server");
	}

	if (boards[board].baseaddress == -1)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board does not exist on server");

	*numchan = numchans[board];
	return DALI_ERR_NONE;
}

UINT CDali::GetChannelInfo(UINT board, UINT chan, DALICHANNEL* chaninfo)
{
	if (board < 0 || board >= numboards)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board handle out of range");

	if (boards[board].baseaddress == -1)
	{
		if (UpdateServerInfo())
			return SetLastError(DALI_ERR_CONNECTIONFAIL, "Could not connect to server");
	}
	
	if (boards[board].baseaddress == -1)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board does not exist on server");

	memcpy(chaninfo, chans[board] + chan, sizeof(DALICHANNEL));	
	return DALI_ERR_NONE;
}

//This function handles all the initialization required when adding a new interface
UINT CDali::SetupNewInterface(DALI_INTERFACE_SETTINGS* dali_int, void* user_hook, DALI_INTERFACE** di)
{
	if (server[0] == 0)
		return SetLastError(DALI_ERR_COULDNOTCONNECT, "No server selected");

	if (dali_int == NULL || user_hook == NULL)
		return SetLastError(DALI_ERR_INVALID_PARAM, "Parameters cannot be NULL");

	//if (dali_int->operation_type < DALI_CMD_AD_SAMPLE || dali_int->operation_type > DALI_CMD_DIO_READBACK)
	//	return SetLastError(DALI_ERR_INVALID_PARAM, "operation_type must be of DALI_CMD_*");

	if (dali_int->dali_board >= numboards)
		return SetLastError(DALI_ERR_INVALID_PARAM, "Invalid dali_board handle");

	//find the next empty space
	int i;
	for (i = 0; i < DALI_MAX_INTERFACES; i++)
	{
		if (!ifaces[i].registered)
			break;
	}

	if (i == DALI_MAX_INTERFACES)
		return SetLastError(DALI_ERR_NOFREEINTERFACE, "Too many interfaces are connected to this class");

	//if the interface is an "alarm" interface, perform the necessary initialization
	ifaces[i].alarm_handle[0] = 0;
	int old_optype = dali_int->operation_type;
	if (dali_int->operation_type & DALI_CMD_ALARM)
	{
		dali_int->operation_type &= ~DALI_CMD_ALARM;  //take out the alarm flag
		env.service = board_soaps[dali_int->dali_board];
		
		if (my_port == -1) //if we haven't initialized a listener, do so now, and find out which port was bound.
		{			
			my_port = dali_int->port;

			while (DALIInitAlarmListener(&env, my_port) != DALI_OK)
			{
				my_port++;
				if (my_port == (dali_int->port + 100))
					return SetLastError(DALI_ERR_COULDNOTPERFORM, "Could not bind to a local port");
			}

		}

		//set up the dali_int struct
		dali_int->port = my_port;

		sprintf(dali_int->alarm.actiontarget, "http://%s:%d/dali/soap/%s", my_ip, my_port, boards[dali_int->dali_board].board);
		strcpy(dali_int->alarm.actiontype, DALI_ACTION_SOAP);
		sprintf(dali_int->alarm.message, "CDali alarm handler");
		
		//now call the appropriate dalic alarm registration
		int rc;
		switch (dali_int->operation_type)
		{
		case DALI_CMD_AD_SAMPLE:
			rc = DALIAlarmADOutOfRange(&env, dali_int->low_channel_port, dali_int->low_value, dali_int->high_value, &dali_int->adsettings, &dali_int->alarm, ifaces[i].alarm_handle, DALI_MAX_ALARM_ID_LEN);
			break;
		case DALI_CMD_DA_READBACK:
			rc = DALIAlarmDAOnChange(&env, dali_int->low_channel_port, &dali_int->alarm, ifaces[i].alarm_handle, DALI_MAX_ALARM_ID_LEN);
			break;
		case DALI_CMD_DIO_IN_BIT:
			rc = DALIAlarmDIOInOnChange(&env, dali_int->low_channel_port, dali_int->high_channel_bit, &dali_int->alarm, ifaces[i].alarm_handle, DALI_MAX_ALARM_ID_LEN);
			break;
		case DALI_CMD_DIO_READBACK_BIT:
			rc = DALIAlarmDIOOutOnChange(&env, dali_int->low_channel_port, dali_int->high_channel_bit, &dali_int->alarm, ifaces[i].alarm_handle, DALI_MAX_ALARM_ID_LEN);
			break;
		}
		if (rc != DALI_OK)
			return SetLastError(DALI_ERR_COULDNOTPERFORM, rc == DALI_CONNECTION_FAILED ? "Could not connect to server" : env.last_error);

		//if we haven't started the alarm thread, start it now
		if (!run_at)
		{		
			at_handle = CreateThread(NULL, 0, ATThread, this, 0, &at_thread_id);
		}
	}

	//fill out the ifaces [interfaces] index 
	ifaces[i].active = 1;
	ifaces[i].registered = 1;
	ifaces[i].handle = i;
	 
	//assign the spread member based on spread flag and op type
	if (dali_int->operation_type & DALI_CMD_SPREAD)
	{		
		ifaces[i].spread = dali_int->high_channel_bit - dali_int->low_channel_port + 1;
		if ((dali_int->operation_type & 0x0030) == 0x0030) ifaces[i].spread = 8;
	}
	else ifaces[i].spread = 0;
	
	*di = &ifaces[i];

	memcpy(&((*di)->settings), dali_int, sizeof(DALI_INTERFACE_SETTINGS));

	(*di)->service = board_soaps[dali_int->dali_board];
	(*di)->settings.operation_type &= ~DALI_CMD_SPREAD;

	dali_int->operation_type = old_optype;

	return DALI_ERR_NONE;
}

//The following three RegisterInterface functions are the shells for creating a new interface depending
//on the parameters used (i.e. CWnd vs. callback function)
UINT CDali::RegisterInterface(DALI_INTERFACE_SETTINGS* dali_int, CWnd** user_cwnd, UINT* int_handle)
{
	if (!alive)
		return SetLastError(DALI_ERR_DECONSTRUCTING, "The CDali object is deconstructing");

	DALI_INTERFACE* di;
	int res = SetupNewInterface(dali_int, user_cwnd, &di);
	if (res) return res;

	for (UINT i = 0; i < di->spread; i++)
		di->user_cwnd[i] = user_cwnd[i];

	if (di->spread == 0)
		di->user_cwnd[0] = *user_cwnd;
	
	di->user_func = NULL;

	*int_handle = di->handle;
	ActivateInterface(di);

	return DALI_ERR_NONE;
}

UINT CDali::RegisterInterface(DALI_INTERFACE_SETTINGS* dali_int, CWnd* user_cwnd, UINT* int_handle)
{
	if (!alive)
		return SetLastError(DALI_ERR_DECONSTRUCTING, "The CDali object is deconstructing");

	DALI_INTERFACE* di;
	int res = SetupNewInterface(dali_int, user_cwnd, &di);
	if (res) return res;

	for (UINT i = 0; i < di->spread; i++)
		di->user_cwnd[i] = &user_cwnd[i];

	if (di->spread == 0)
		di->user_cwnd[0] = user_cwnd;
	
	di->user_func = NULL;

	*int_handle = di->handle;
	ActivateInterface(di);

	return DALI_ERR_NONE;
}

UINT CDali::RegisterInterface(DALI_INTERFACE_SETTINGS* dali_int, DALI_CALLBACKFUNC user_func, UINT* int_handle)
{
	if (!alive)
		return SetLastError(DALI_ERR_DECONSTRUCTING, "The CDali object is deconstructing");

	DALI_INTERFACE* di;
	int res = SetupNewInterface(dali_int, user_func, &di);
	if (res) return res;

	di->user_cwnd[0] = NULL;
	di->user_func = user_func;

	*int_handle = di->handle;
	ActivateInterface(di);

	return DALI_ERR_NONE;
}

//This function tells the control on the other end of the interface that it is being activated.
//It also sets up the interface in the priority queue.
void CDali::ActivateInterface(DALI_INTERFACE* di)
{
	//if (di->alarm_handle != -1)
		//return;

	DALI_DATA data;

	data.lpCDALI = this;
	data.low_channel_port = di->settings.low_channel_port;
	data.high_channel_bit = di->settings.high_channel_bit;
	data.operation_type = di->settings.operation_type;
	data.dali_board = di->settings.dali_board;

	//output interfaces do not require anything to be placed in the thread
	//since they are event driven, not timer driven.  so we just return from the
	//function.
	if (di->settings.operation_type == DALI_CMD_DA_CONVERT ||
		di->settings.operation_type == DALI_CMD_DIO_OUT_BYTE ||
		di->settings.operation_type == DALI_CMD_DIO_OUT_BIT)
	{
		SendData(di, DALI_CMD_REGISTERED, &data);
		return;
	}
		
	//otherwise we need to put the interface into the PQ and initialize the PQ thread
	//if it has not been started yet.
	if (di->alarm_handle[0] == 0)
	{
		numifaces++;

		dalipq.InsertInterface(di);

		if (numifaces == 1)
		{
			run_pq = 1;
			pq_handle = CreateThread(NULL, 0, PQThread, this, 0, &pq_thread_id);
		}
	}
	
	//SendData will just send the REGISTERED flag to the control along with the data struct
	SendData(di, DALI_CMD_REGISTERED, &data);
	
}

UINT CDali::UnRegisterInterface(UINT handle)
{	
	if (handle >= DALI_MAX_INTERFACES)
		return SetLastError(DALI_ERR_INVALID_PARAM, "Board handle out of range");

	if (!ifaces[handle].registered)
		return SetLastError(DALI_ERR_INVALID_PARAM, "Handle not registered");

	ifaces[handle].active = 0;
	ifaces[handle].registered = 0;

	if (ifaces[handle].alarm_handle[0] != 0)
	{
		if (DALIUnregisterAlarm(&env, ifaces[handle].alarm_handle))
			return SetLastError(DALI_ERR_COULDNOTPERFORM, "Could not unregister alarm");
	}

	return DALI_ERR_NONE;
}


UINT CDali::PerformADSample(UINT board, UINT channel, DALIADSETTINGS* adsettings, int* value)
{
	if (!alive)
		return SetLastError(DALI_ERR_DECONSTRUCTING, "The CDali object is deconstructing");

	if (board < 0 || board >= numboards)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board handle out of range");
	
	env.service = board_soaps[board];
	int ret = DALIADSample(&env, channel, adsettings, value);
	
	if (ret)
		return SetLastError(DALI_ERR_COULDNOTPERFORM, ret == DALI_CONNECTION_FAILED ? "Could not connect to server" : env.last_error);
	
	return DALI_ERR_NONE;
}

UINT CDali::PerformADScan(UINT board, UINT low_channel, UINT high_channel, DALIADSETTINGS* adsettings, int* values)
{
	if (!alive)
		return SetLastError(DALI_ERR_DECONSTRUCTING, "The CDali object is deconstructing");

	if (board < 0 || board >= numboards)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board handle out of range");
	
	env.service = board_soaps[board];
	int ret = DALIADScan(&env, low_channel, high_channel, adsettings, values);

	if (ret)
		return SetLastError(DALI_ERR_COULDNOTPERFORM, ret == DALI_CONNECTION_FAILED ? "Could not connect to server" : env.last_error);

	return DALI_ERR_NONE;
}

UINT CDali::PerformADSampleMulti(UINT board, UINT channel, UINT count, float hz, DALIADSETTINGS* adsettings, int* value)
{
	if (!alive)
		return SetLastError(DALI_ERR_DECONSTRUCTING, "The CDali object is deconstructing");

	if (board < 0 || board >= numboards)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board handle out of range");
	
	env.service = board_soaps[board];
	int ret = DALIADSampleMulti(&env, channel,count, hz, adsettings, value);

	if (ret)
		return SetLastError(DALI_ERR_COULDNOTPERFORM, ret == DALI_CONNECTION_FAILED ? "Could not connect to server" : env.last_error);

	return DALI_ERR_NONE;
}

UINT CDali::PerformADScanMulti(UINT board, UINT low_channel, UINT high_channel, UINT count, float hz, DALIADSETTINGS* adsettings, int* values)
{
	if (!alive)
		return SetLastError(DALI_ERR_DECONSTRUCTING, "The CDali object is deconstructing");

	if (board < 0 || board >= numboards)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board handle out of range");
	
	env.service = board_soaps[board];
	int ret = DALIADScanMulti(&env, low_channel, high_channel, count, hz, adsettings, values);

	if (ret)
		return SetLastError(DALI_ERR_COULDNOTPERFORM, ret == DALI_CONNECTION_FAILED ? "Could not connect to server" : env.last_error);

	return DALI_ERR_NONE;
}

UINT CDali::PerformDAConvert(UINT board, UINT channel, int value)
{
	if (!alive)
		return SetLastError(DALI_ERR_DECONSTRUCTING, "The CDali object is deconstructing");

	if (board < 0 || board >= numboards)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board handle out of range");
	
	env.service = board_soaps[board];
	int ret = DALIDAConvert(&env, channel, value);

	if (ret)
		return SetLastError(DALI_ERR_COULDNOTPERFORM, ret == DALI_CONNECTION_FAILED ? "Could not connect to server" : env.last_error);

	return DALI_ERR_NONE;
}

UINT CDali::PerformDAReadBack(UINT board, UINT channel, int* value)
{
	if (!alive)
		return SetLastError(DALI_ERR_DECONSTRUCTING, "The CDali object is deconstructing");

	if (board < 0 || board >= numboards)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board handle out of range");
	
	env.service = board_soaps[board];
	int ret = DALIDAReadback(&env, channel, value);

	if (ret)
		return SetLastError(DALI_ERR_COULDNOTPERFORM, ret == DALI_CONNECTION_FAILED ? "Could not connect to server" : env.last_error);

	return DALI_ERR_NONE;
}

UINT CDali::PerformDIOOutputByte(UINT board, UINT channel, int value)
{
	if (!alive)
		return SetLastError(DALI_ERR_DECONSTRUCTING, "The CDali object is deconstructing");

	if (board < 0 || board >= numboards)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board handle out of range");
	
	env.service = board_soaps[board];
	int ret = DALIDIOOutputByte(&env, channel, value);

	if (ret)
		return SetLastError(DALI_ERR_COULDNOTPERFORM, ret == DALI_CONNECTION_FAILED ? "Could not connect to server" : env.last_error);

	return DALI_ERR_NONE;
}

UINT CDali::PerformDIOOutputBit(UINT board, UINT channel, UINT bit, int value)
{
	if (!alive)
		return SetLastError(DALI_ERR_DECONSTRUCTING, "The CDali object is deconstructing");

	if (board < 0 || board >= numboards)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board handle out of range");
	
	env.service = board_soaps[board];
	int ret = DALIDIOOutputBit(&env, channel, bit, value);

	if (ret)
		return SetLastError(DALI_ERR_COULDNOTPERFORM, ret == DALI_CONNECTION_FAILED ? "Could not connect to server" : env.last_error);

	return DALI_ERR_NONE;
}

UINT CDali::PerformDIOInputByte(UINT board, UINT channel, int* value)
{
	if (!alive)
		return SetLastError(DALI_ERR_DECONSTRUCTING, "The CDali object is deconstructing");

	if (board < 0 || board >= numboards)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board handle out of range");
	
	env.service = board_soaps[board];
	int ret = DALIDIOInputByte(&env, channel, value);

	if (ret)
		return SetLastError(DALI_ERR_COULDNOTPERFORM, ret == DALI_CONNECTION_FAILED ? "Could not connect to server" : env.last_error);

	return DALI_ERR_NONE;
}

UINT CDali::PerformDIOInputBit(UINT board, UINT channel, UINT bit, int* value)
{
	if (!alive)
		return SetLastError(DALI_ERR_DECONSTRUCTING, "The CDali object is deconstructing");

	if (board < 0 || board >= numboards)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board handle out of range");
	
	env.service = board_soaps[board];
	int ret = DALIDIOInputBit(&env, channel, bit, value);

	if (ret)
		return SetLastError(DALI_ERR_COULDNOTPERFORM, ret == DALI_CONNECTION_FAILED ? "Could not connect to server" : env.last_error);

	return DALI_ERR_NONE;
}

UINT CDali::PerformDIOReadBackByte(UINT board, UINT channel, int* value)
{
	if (!alive)
		return SetLastError(DALI_ERR_DECONSTRUCTING, "The CDali object is deconstructing");

	if (board < 0 || board >= numboards)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board handle out of range");
	
	env.service = board_soaps[board];
	int ret = DALIDIOInputByte(&env, channel, value);

	if (ret)
		return SetLastError(DALI_ERR_COULDNOTPERFORM, ret == DALI_CONNECTION_FAILED ? "Could not connect to server" : env.last_error);

	return DALI_ERR_NONE;
}

UINT CDali::PerformDIOReadBackBit(UINT board, UINT channel, UINT bit, int* value)
{
	if (!alive)
		return SetLastError(DALI_ERR_DECONSTRUCTING, "The CDali object is deconstructing");

	if (board < 0 || board >= numboards)
		return SetLastError(DALI_ERR_INVALID_BOARD, "Board handle out of range");
	
	env.service = board_soaps[board];
	int ret = DALIDIOInputBit(&env, channel, bit, value);

	if (ret)
		return SetLastError(DALI_ERR_COULDNOTPERFORM, ret == DALI_CONNECTION_FAILED ? "Could not connect to server" : env.last_error);

	return DALI_ERR_NONE;
}

//Ping the server (and set the "connection established" bit accordingly.)
UINT CDali::Ping()
{
	env.service = server_soap;
	int res;
	int ret = DALIPing(&env, &res);

	SetConnEst((ret == 0) ? 1 : 0);

	if (ret)
		return SetLastError(DALI_ERR_COULDNOTPERFORM, ret == DALI_CONNECTION_FAILED ? "Could not connect to server" : env.last_error);

	return DALI_ERR_NONE;
}

void CDali::DeleteChannelInfo()
{
	for (int x = 0; x < DALI_MAX_BOARDS; x++)
	{
		if (chans[x] != NULL) 
			delete [] chans[x];

		chans[x] = NULL;
		numchans[x] = 0;
	}
}

UINT CDali::SetLastError(UINT code, LPCTSTR errorstr)
{
	strcpy(last_error, errorstr);
	return code;
}

//This is the thread function to process the commands waiting in the priority queue
void CDali::ProcessPQ()
{
	while (run_pq) //only run while run_pq != 0
	{
		dalipq.Lock(); //lock the PQ - this is used because we want to keep the PQ safe
		               //if the main thread is adding a new interface at the same time.

		ULONG curtime = GetTickCount();

		DALI_INTERFACE* next = dalipq.GetFirst(); //peek at the first (soonest) event
		if (curtime > next->next_trigger) //if it's time for the event...
		{
			PerformInterface(next); //Perform the the event
			dalipq.PushFirst();	    //and push it back into the queue
		}

		dalipq.UnLock();  //unlock

		Sleep(10); //this is so that the PQ thread does not hog resources
		if (numifaces == 0)
			run_pq = 0; //kill the thread if there are no interfaces left
	}

	//run_pq = 1;
}

//This is basically an interface dispatch.  It will perform whatever DALI function is associated with the
//interface
void CDali::PerformInterface(DALI_INTERFACE* di)
{
	DALI_DATA data;
	int out_code = di->settings.operation_type;
	UINT return_code = DALI_ERR_NONE;
	DALI_INTERFACE_SETTINGS* sett = &(di->settings);

	data.low_channel_port = sett->low_channel_port;
	data.high_channel_bit = sett->high_channel_bit;
	data.operation_type = sett->operation_type;
	data.dali_board = sett->dali_board;
	data.lpCDALI = this;
	data.interface_handle = di->handle;

	switch (di->settings.operation_type)
	{
	case DALI_CMD_AD_SAMPLE:
				
		
		return_code = PerformADSample(sett->dali_board, sett->low_channel_port, &sett->adsettings, data.values);
		break;
	
	case DALI_CMD_AD_SCAN:
		
		return_code = PerformADScan(sett->dali_board, sett->low_channel_port, sett->high_channel_bit, &sett->adsettings, data.values);
		break;

	case DALI_CMD_DA_READBACK:
		
		return_code = PerformDAReadBack(sett->dali_board, sett->low_channel_port, data.values);
		break;

	case DALI_CMD_DIO_IN_BYTE:

		return_code = PerformDIOInputByte(sett->dali_board, sett->low_channel_port, data.values);
		break;

	case DALI_CMD_DIO_IN_BIT:

		return_code = PerformDIOInputBit(sett->dali_board, sett->low_channel_port, sett->high_channel_bit, data.values);
		break;

	case DALI_CMD_DIO_READBACK_BYTE:

		return_code = PerformDIOReadBackByte(sett->dali_board, sett->low_channel_port, data.values);
		break;

	case DALI_CMD_DIO_READBACK_BIT:

		return_code = PerformDIOReadBackBit(sett->dali_board, sett->low_channel_port, sett->high_channel_bit, data.values);
		break;
	}

	if (return_code == DALI_ERR_DECONSTRUCTING)
		return;


	//All of the SetConnEst business is used to inform all other interfaces that this one has failed.
	//Basically, all of the associated buttons will become "disabled" until an interfaces successfully
	//communicates with the server again (or it is pinged.)
	data.error = return_code;
	if (return_code)
	{
		out_code = DALI_CMD_ERROR;
		strcpy(data.error_str, GetLastErrorString());		
		if (conn_est)
			SetConnEst(0);
	}
	else if (!conn_est)
		SetConnEst(1);
	
	SendData(di, out_code, &data);

}

//This function sends data to the other end of the interface
void CDali::SendData(DALI_INTERFACE* di, int code, DALI_DATA* data)
{
	if (di->user_func)
	{
		di->user_func(code, data);
	}
	else 
	{
		if (di->spread == 0)
		{
			if (!alive) return;	
			di->user_cwnd[0]->SendMessage(DALI_NOTIFY, (WPARAM)code, (LPARAM)data);
		}
		//Things only get complicated if we're dealing with a spread interface
		else
		{
			int sent = 0;			

			//Reset the code so that the interface controls aren't confused
			//They think they're getting normal data, not a spread
			if (code == DALI_CMD_AD_SCAN) code = DALI_CMD_AD_SAMPLE;
			if (code == DALI_CMD_DIO_IN_BYTE) code = DALI_CMD_DIO_IN_BIT;
			if (code == DALI_CMD_DIO_READBACK_BYTE) code = DALI_CMD_DIO_READBACK_BIT;

			UINT r_low_channel_port = data->low_channel_port;
			int value0 = data->values[0];
			
			//So send the data to the appropriate CWnds/callbacks
			for (UINT i = 0; i < di->spread; i++)
			{
				if (di->user_cwnd[i] != NULL)
				{
					if (code == DALI_CMD_AD_SAMPLE) 
					{
						data->low_channel_port = r_low_channel_port + i;
						data->values[0] = data->values[i];
					}
					if (code == DALI_CMD_DIO_IN_BIT || code == DALI_CMD_DIO_READBACK_BIT) 
					{
						data->high_channel_bit = i;
						data->values[0] = (value0 >> i) & 1;
					}
					
					if (!alive) return;		
					di->user_cwnd[i]->SendMessage(DALI_NOTIFY, (WPARAM)code, (LPARAM)data);
					sent = 1;
				}
			}

			//if (!sent)
			//	di->active = 0;
		}
	}	
}

//This is the thread for monitoring the alarm notifications from the server
void CDali::AlertThread()
{
	if (run_at)
		return;

	run_at = 1;

	DALIALARMNOTICE dai;

	while (run_at) //execute while run_at != 0
	{			
		int rc = DALIWaitForAlarm(&env, &dai); //this will block until notification arrives
		                                       //or the socket is closed

		if (!alive) //a shortcut - in the deconstructor we set alive = 0, so this will return immediately
			return; //if we are desconstructing

		if (rc == DALI_TCP_BIND_ERROR)
		{
			Sleep(10); //on a bind error, sleep and try again.
			continue;
		}

		if (rc == DALI_TCP_ACCEPT_ERROR)
		{
			run_at = 0;	//accept error means the socket has closed, so kill the thread.
			return;
		}
		
		//now we're going to search for the alarm and process it.
		BOOL found = 0; //has the alarm been found in our list?
		for (UINT i = 0; i < DALI_MAX_INTERFACES; i++)
		{
			if (!strcmp(ifaces[i].alarm_handle, dai.id)) //compare alarm handles
			{
				DALI_INTERFACE* di = &ifaces[i];
				DALI_DATA data;
				int out_code = di->settings.operation_type;
				DALI_INTERFACE_SETTINGS* sett = &(di->settings);

				data.low_channel_port = sett->low_channel_port;
				data.high_channel_bit = sett->high_channel_bit;
				data.operation_type = sett->operation_type;
				data.dali_board = sett->dali_board;
				data.lpCDALI = this;
				data.interface_handle = di->handle;
				data.values[0] = dai.value;

				if (di->active)
					SendData(di, out_code, &data); //send the alarm data to the interface control

				found = 1;
			}
		}

		if (!found) //this means the alarm is for something we didn't register with
		{           //so there must be an error on the server, or something went wrong..
			DALIUnregisterAlarm(&env, dai.id); //so just unregister the alarm.
		}
	}
	
}

//This tries to deal with the complicated issue of reconnection when the network goes down.
//We want to disable/enable the controls to match the connection status, and we also want
//to reregister the alarms because the server has either been restarted, or has unregistered
//the alarms itself because of timeout.  
void CDali::SetConnEst(int c)
{
	if (conn_est == c) //If we're setting the state to where it already is, return.
		return;

	conn_est = c;

	DALI_DATA data;
	data.lpCDALI = this;
	data.error = DALI_ERR_CONNECTIONFAIL;
	sprintf(data.error_str, "Connection failed");

	int reregalarms = 1;

	for (UINT i = 0; i < DALI_MAX_INTERFACES; i++) //going through all the interfaces..
	{
		DALI_INTERFACE* di = &ifaces[i];

		if (!di->registered) //skip unregistered interfaces
			continue;
		
		DALI_INTERFACE_SETTINGS* sett = &(di->settings);

		data.low_channel_port = sett->low_channel_port;
		data.high_channel_bit = sett->high_channel_bit;
		data.operation_type = sett->operation_type;
		data.dali_board = sett->dali_board;
		data.interface_handle = di->handle;		

		SendData(di, c ? DALI_CMD_REGISTERED : DALI_CMD_ERROR, &data); //tell the control that we're back up
		                                                               //or that we went down..

		if (c && (di->alarm_handle[0] != 0) && (reregalarms != 0))
		{
			env.service = board_soaps[sett->dali_board];
			
			//now we're going to check if we need to register the alarms by seeing if the
			//server knows about our existing alarms already
			if ((reregalarms == 1) && !DALIAlarmIsAlive(&env, ifaces[i].alarm_handle))
			{
				reregalarms = 0; //this means the server has our alarms, so it must have been a
				continue;        //temporary failure.. don't check any other alarms.
			}

			reregalarms = 2;
				
			//reregister alarm
			int rc;
			switch (sett->operation_type)
			{
			case DALI_CMD_AD_SAMPLE:
				rc = DALIAlarmADOutOfRange(&env, sett->low_channel_port, sett->low_value, sett->high_value, &sett->adsettings, &sett->alarm, ifaces[i].alarm_handle, DALI_MAX_ALARM_ID_LEN);
				break;
			case DALI_CMD_DA_READBACK:
				rc = DALIAlarmDAOnChange(&env, sett->low_channel_port, &sett->alarm, ifaces[i].alarm_handle, DALI_MAX_ALARM_ID_LEN);
				break;
			case DALI_CMD_DIO_IN_BIT:
				rc = DALIAlarmDIOInOnChange(&env, sett->low_channel_port, sett->high_channel_bit, &sett->alarm, ifaces[i].alarm_handle, DALI_MAX_ALARM_ID_LEN);
				break;
			case DALI_CMD_DIO_READBACK_BIT:
				rc = DALIAlarmDIOOutOnChange(&env, sett->low_channel_port, sett->high_channel_bit, &sett->alarm, ifaces[i].alarm_handle, DALI_MAX_ALARM_ID_LEN);
				break;
			}
			//if (rc != DALI_OK)
			//	return SetLastError(DALI_ERR_COULDNOTPERFORM, rc == DALI_CONNECTION_FAILED ? "Could not connect to server" : env.last_error);

		}
	}

}